use serde::{Deserialize, Serialize};
use std::collections::BTreeMap;
use std::sync::Arc;
use std::sync::Mutex;
use uuid::Uuid;

#[derive(Debug, Serialize, Deserialize)]
pub struct NewPost {
    title: String,
    content: String,
}

#[derive(Debug, Serialize, Deserialize, Clone)]
pub struct Post {
    pub uuid: Uuid,
    pub title: String,
    pub content: String,
    pub votes: VoteCount,
}

impl From<NewPost> for Post {
    fn from(post: NewPost) -> Self {
        Self {
            uuid: Uuid::new_v4(),
            title: post.title,
            content: post.content,
            votes: VoteCount::new(),
        }
    }
}

#[derive(Clone)]
pub struct AppState {
    pub posts: Arc<Mutex<BTreeMap<Uuid, Post>>>,
}

impl AppState {
    pub fn new() -> Self {
        Self {
            posts: Arc::new(Mutex::new(BTreeMap::new())),
        }
    }
}

// Part of the post struct
#[derive(Debug, Serialize, Deserialize, Clone)]
pub struct VoteCount {
    pub up: u32,
    pub down: u32,
}

impl VoteCount {
    fn new() -> Self {
        Self { up: 0, down: 0 }
    }
}
